"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = Param;

var React = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

/**
 * @class A placeholder for a replacement parameter in the body of a Translate
 * component.
 *
 * This component renders into the value of the named parameter to the Translate
 * component. Children are not allowed in this component and typically, it is used with
 * the self-closing syntax.
 *
 * @example
 * <pre>
 *   <Translate id="x" description="y">
 *     The file <Param value={filelist[i].path} description="Name of the file that was deleted."/> has been deleted.
 *   </Translate>
 * </pre>
 */
function Param(props) {
  var _ref = props || {},
      value = _ref.value,
      name = _ref.name,
      wrapper = _ref.wrapper,
      className = _ref.className;

  var ret;

  switch (_typeof(value)) {
    default:
    case 'undefined':
      ret = '';
      break;

    case 'boolean':
    case 'number':
      ret = String(value);
      break;

    case 'function':
      ret = value();
      break;

    case 'string':
      ret = value;
      break;

    case 'object':
      if (value === null) {
        ret = '';
      } else if (React.isValidElement(value)) {
        ret = value;
      } else {
        ret = value.toString();
      }

      break;
  }

  if (wrapper) {
    var wrapperName = name || wrapper;
    return React.createElement(wrapper, {
      key: wrapperName,
      name: wrapperName,
      className: className
    }, ret);
  } else {
    return ret;
  }
}

;
Param.propTypes = {
  /** The name of this parameter */
  "name": _propTypes["default"].string.isRequired,

  /** The value of this parameter */
  "value": _propTypes["default"].any,

  /** Optional: A description of this parameter to help the translators understand what it is */
  "description": _propTypes["default"].string,

  /** Optional: wrap the output in this html tag */
  "wrapper": _propTypes["default"].string,

  /** Optional: if the output is wrapped in an html tag, use this as the CSS class name */
  "className": _propTypes["default"].string
};
//# sourceMappingURL=Param.js.map
