"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _react = _interopRequireDefault(require("react"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var MessageAccumulator = require('message-accumulator')["default"]; // ES5 CommonJS module


var Node = require('ilib-tree-node')["default"]; // ES5 CommonJS module

/**
 * @class Compose a tree of React elements into a single string.
 *
 * @param {React.Element} element the element to compose
 */


var Composition =
/*#__PURE__*/
function () {
  function Composition(element) {
    _classCallCheck(this, Composition);

    this.element = element;
    this.isComposed = false;
    this.ma = new MessageAccumulator();
  }

  _createClass(Composition, [{
    key: "recompose",
    value: function recompose(element) {
      var _this = this;

      switch (_typeof(element)) {
        case 'object':
          if (Array.isArray(element)) {
            element.forEach(function (subelement) {
              return _this.recompose(subelement);
            });
          } else if (element) {
            if (element.type === 'Param') {
              this.ma.addParam(element);
            } else {
              this.ma.push(element);

              _react["default"].Children.forEach(element.props.children, function (child) {
                return _this.recompose(child);
              });

              this.ma.pop();
            }
          }

          break;

        case 'number':
        case 'boolean':
          this.ma.addText(String(element));
          break;

        case 'string':
          this.ma.addText(element);
          break;

        default:
          break;
      }
    }
    /**
     * Compose a tree of react elements to a string that can be translated.
     *
     * @return {string} a string representing the tree of react elements
     */

  }, {
    key: "compose",
    value: function compose() {
      this.index = 0;

      if (!this.isComposed) {
        this.recompose(this.element);
      }

      this.isComposed = true;
      return this.ma.getMinimalString();
    }
    /**
     * @private
     */

  }, {
    key: "mapToReactElements",
    value: function mapToReactElements(node) {
      if (!node) return '';
      var children = [];

      for (var i = 0; i < node.children.length; i += 1) {
        children.push(this.mapToReactElements(node.children[i]));
      }

      var el = node.extra;

      if (children.length === 0 && el && el.props) {
        children = el.props.children;
      }

      if (children && children.length === 1 && typeof children[0] === 'string') {
        children = children[0];
      }

      if (el) {
        return children && children.length ? _react["default"].cloneElement(el, {
          key: el.key
        }, children) : _react["default"].cloneElement(el, {
          key: el.key
        });
      }

      if (children.length) {
        return children;
      }

      return node.value || '';
    }
    /**
     * Convert a composed string back into an array of React elements. The elements are clones of
     * the same ones that this composition was created with, so that they have the same type and
     * props and such as the originals. The elements may be re-ordered from the original, however,
     * if the grammar of the target language requires moving around text, HTML tags, or
     * subcomponents.
     *
     * @param {string} string the string to decompose into a tree of React elements.
     * @return {React.Element} a react element
     */

  }, {
    key: "decompose",
    value: function decompose(string) {
      if (!this.isComposed) {
        // need to create the mapping first from names to react elements
        this.compose();
      }

      var translation = MessageAccumulator.create(string, this.ma);
      var nodeArray = this.ma.getPrefix().concat(translation.root.toArray().slice(1, -1)).concat(this.ma.getSuffix()); // convert to a tree again

      return this.mapToReactElements(Node.fromArray(nodeArray));
    }
  }]);

  return Composition;
}();

exports["default"] = Composition;
;
//# sourceMappingURL=Composition.js.map
